# Generated by Django 2.0 on 2019-08-15 13:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('security', '0011_auto_20190815_1508'),
    ]

    operations = [
        migrations.AlterField(
            model_name='commandlog',
            name='changed_at',
            field=models.DateTimeField(auto_now=True, db_index=True, verbose_name='changed at'),
        ),
        migrations.AlterField(
            model_name='commandlog',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, db_index=True, verbose_name='created at'),
        ),
        migrations.AlterField(
            model_name='inputloggedrequest',
            name='changed_at',
            field=models.DateTimeField(auto_now=True, db_index=True, verbose_name='changed at'),
        ),
        migrations.AlterField(
            model_name='inputloggedrequest',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, db_index=True, verbose_name='created at'),
        ),
        migrations.AlterField(
            model_name='inputloggedrequest',
            name='path',
            field=models.CharField(blank=True, db_index=True, max_length=255, verbose_name='URL path'),
        ),
        migrations.AlterField(
            model_name='inputloggedrequest',
            name='slug',
            field=models.CharField(blank=True, db_index=True, max_length=255, null=True, verbose_name='slug'),
        ),
        migrations.AlterField(
            model_name='outputloggedrequest',
            name='changed_at',
            field=models.DateTimeField(auto_now=True, db_index=True, verbose_name='changed at'),
        ),
        migrations.AlterField(
            model_name='outputloggedrequest',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, db_index=True, verbose_name='created at'),
        ),
        migrations.AlterField(
            model_name='outputloggedrequest',
            name='path',
            field=models.CharField(blank=True, db_index=True, max_length=255, verbose_name='URL path'),
        ),
        migrations.AlterField(
            model_name='outputloggedrequest',
            name='slug',
            field=models.CharField(blank=True, db_index=True, max_length=255, null=True, verbose_name='slug'),
        ),
    ]
