# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2019-08-15 18:12
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ('reversion_log', '0002_auto_20180418_0925'),
    ]

    operations = [
        migrations.AlterField(
            model_name='inputrequestrevision',
            name='logged_request',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                    related_name='input_logged_request_revisions', to='security.InputLoggedRequest',
                                    verbose_name='logged request'),
        ),
        migrations.AlterField(
            model_name='inputrequestrevision',
            name='revision',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE,
                                       related_name='input_logged_request_revision', to='reversion.Revision',
                                       verbose_name='revision'),
        ),
    ]
