# -*- coding: utf-8 -*-
# Generated by Django 1.11.24 on 2019-11-03 22:05
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('security', '0016_migration'),
    ]

    operations = [
        migrations.AlterField(
            model_name='celerytasklog',
            name='celery_task_id',
            field=models.CharField(max_length=250, verbose_name='celery ID'),
        ),
        migrations.AlterUniqueTogether(
            name='celerytasklog',
            unique_together=set([('celery_task_id', 'retries')]),
        ),
    ]
