# -*- coding: utf-8 -*-
# Generated by Django 1.11.26 on 2019-11-26 15:29
from __future__ import unicode_literals

from django.db import migrations, models
import enumfields.fields
import security.enums


class Migration(migrations.Migration):

    dependencies = [
        ('security', '0017_migration'),
    ]

    operations = [
        migrations.AlterField(
            model_name='inputloggedrequest',
            name='ip',
            field=models.GenericIPAddressField(db_index=True, verbose_name='IP address'),
        ),
        migrations.AlterField(
            model_name='inputloggedrequest',
            name='response_timestamp',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='response timestamp'),
        ),
        migrations.AlterField(
            model_name='inputloggedrequest',
            name='type',
            field=enumfields.fields.NumEnumField(db_index=True, default=1, enum=security.enums.InputLoggedRequestType,
                                                 verbose_name='type'),
        ),
        migrations.AlterField(
            model_name='outputloggedrequest',
            name='response_timestamp',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='response timestamp'),
        ),
    ]
